#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// Modified to display as Histogram, 20-Oct-2007, KBJ.

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// The Commodity Channel Index (CCI) measures the variation of a security's price from its statistical mean. High values show that prices are unusually high compared to average prices whereas low values indicate that prices are unusually low.
    /// </summary>
    [Description("The Commodity Channel Index (CCI) measures the variation of a security's price from its statistical mean. High values show that prices are unusually high compared to average prices whereas low values indicate that prices are unusually low.")]
    [Gui.Design.DisplayName("CCI_Histogram (Commodity Channel Index)")]
    public class CCI_Histogram : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int period = 14; // Default setting for Period
        // User defined variables (add any user defined variables below)
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(new Pen(Color.Blue, 3), PlotStyle.Line, "CCImiddle"));			// Between margins.
            Add(new Plot(new Pen(Color.DarkGreen, 3), PlotStyle.Line, "CCIoverbought"));// Upper = Overbought (>+100)
            Add(new Plot(new Pen(Color.DarkRed, 3), PlotStyle.Line, "CCIoversold"));	// Lower = Oversold   (<-100)
			Add(new Plot(new Pen(Color.Green, 5), PlotStyle.Bar, "CCIbarHi"));			// Histogram above zero.
			Add(new Plot(new Pen(Color.Red, 5), PlotStyle.Bar, "CCIbarLo"));			// Histogram below zero.
			Add(new Line(Color.DarkGray, 200,  "Level2"));
            Add(new Line(Color.DarkGray, 100,  "Level1"));
            Add(new Line(Color.DarkGray, 0,	   "ZeroLine"));
            Add(new Line(Color.DarkGray, -100, "LevelM1"));
            Add(new Line(Color.DarkGray, -200, "LevelM2"));
			Period = 14;
            CalculateOnBarClose	= true;
            Overlay				= false;
            PriceTypeSupported	= true;
			
			// Set the threshold values for each plot
			Plots[0].Min = -100;									// Not oversold or overbought.
			Plots[0].Max = 100;
			Plots[1].Min = 100;										// Overbought is above this.
			Plots[2].Max = -100;									// Oversold is below this.
			Plots[3].Min = 0;										// Overbought is above this.
			Plots[4].Max = 0;										// Oversold is below this.
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			double mean = 0;
			for (int idx = Math.Min(CurrentBar, Period - 1); idx >= 0; idx--)
				mean += Math.Abs(Typical[idx] - SMA(Typical, Period)[0]);
			double plotval = (Typical[0] - SMA(Typical, Period)[0]) / (mean == 0 ? 1 : (0.015 * (mean / Math.Min(Period, CurrentBar + 1))));
			
			// Note, some of the following settings will be ignored based on the Min & Max values defined in Initialize() above:
			CCImiddle.Set( plotval );								// Blue for NOT over-sold or over-bought.
			CCIoverbought.Set( plotval );							// DarkGreen for over-bought.
			CCIoversold.Set( plotval );								// DarkRed for over-sold.
			CCIbarHi.Set( plotval );								// Green histogram for over-bought.
			CCIbarLo.Set( plotval );								// Red histogram for over-sold.
		}

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries CCImiddle
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries CCIoverbought
        {
            get { return Values[1]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries CCIoversold
        {
            get { return Values[2]; }
        }

       [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries CCIbarHi
        {
            get { return Values[3]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries CCIbarLo
        {
            get { return Values[4]; }
        }

		[Description("Numbers of bars used for calculations")]
        [Category("Parameters")]
        public int Period
        {
            get { return period; }
            set { period = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private CCI_Histogram[] cacheCCI_Histogram = null;

        private static CCI_Histogram checkCCI_Histogram = new CCI_Histogram();

        /// <summary>
        /// The Commodity Channel Index (CCI) measures the variation of a security's price from its statistical mean. High values show that prices are unusually high compared to average prices whereas low values indicate that prices are unusually low.
        /// </summary>
        /// <returns></returns>
        public CCI_Histogram CCI_Histogram(int period)
        {
            return CCI_Histogram(Input, period);
        }

        /// <summary>
        /// The Commodity Channel Index (CCI) measures the variation of a security's price from its statistical mean. High values show that prices are unusually high compared to average prices whereas low values indicate that prices are unusually low.
        /// </summary>
        /// <returns></returns>
        public CCI_Histogram CCI_Histogram(Data.IDataSeries input, int period)
        {
            checkCCI_Histogram.Period = period;
            period = checkCCI_Histogram.Period;

            if (cacheCCI_Histogram != null)
                for (int idx = 0; idx < cacheCCI_Histogram.Length; idx++)
                    if (cacheCCI_Histogram[idx].Period == period && cacheCCI_Histogram[idx].EqualsInput(input))
                        return cacheCCI_Histogram[idx];

            CCI_Histogram indicator = new CCI_Histogram();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Period = period;
            indicator.SetUp();

            CCI_Histogram[] tmp = new CCI_Histogram[cacheCCI_Histogram == null ? 1 : cacheCCI_Histogram.Length + 1];
            if (cacheCCI_Histogram != null)
                cacheCCI_Histogram.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheCCI_Histogram = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// The Commodity Channel Index (CCI) measures the variation of a security's price from its statistical mean. High values show that prices are unusually high compared to average prices whereas low values indicate that prices are unusually low.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.CCI_Histogram CCI_Histogram(int period)
        {
            return _indicator.CCI_Histogram(Input, period);
        }

        /// <summary>
        /// The Commodity Channel Index (CCI) measures the variation of a security's price from its statistical mean. High values show that prices are unusually high compared to average prices whereas low values indicate that prices are unusually low.
        /// </summary>
        /// <returns></returns>
        public Indicator.CCI_Histogram CCI_Histogram(Data.IDataSeries input, int period)
        {
            return _indicator.CCI_Histogram(input, period);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// The Commodity Channel Index (CCI) measures the variation of a security's price from its statistical mean. High values show that prices are unusually high compared to average prices whereas low values indicate that prices are unusually low.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.CCI_Histogram CCI_Histogram(int period)
        {
            return _indicator.CCI_Histogram(Input, period);
        }

        /// <summary>
        /// The Commodity Channel Index (CCI) measures the variation of a security's price from its statistical mean. High values show that prices are unusually high compared to average prices whereas low values indicate that prices are unusually low.
        /// </summary>
        /// <returns></returns>
        public Indicator.CCI_Histogram CCI_Histogram(Data.IDataSeries input, int period)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.CCI_Histogram(input, period);
        }

    }
}
#endregion
